@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2008, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2009, Live Networks, Inc.  All rights reserved
 // A test program that streams a WAV audio file via RTP/RTCP
 // main program
 
@@ -86,7 +86,7 @@ void play() {
 
   // Add in any filter necessary to transform the data prior to streaming.
   // (This is where any audio compression would get added.)
-  char* mimeType;
+  char const* mimeType;
   unsigned char payloadFormatCode;
   if (bitsPerSample == 16) {
 #ifdef CONVERT_TO_ULAW
